  /**
   * A twitter visualization tool. 
   */

import interfascia.*;

Twitter myTwitter;
float[] times;
float[][] score;
ArrayList persons,msgs,dates;
float  maxtime, mintime;
int Nmax = 100;
int maxPages = 10;
PFont font;
String searchString;
boolean loading;
int pageLimit;
int showThreshold;

GUIController c;
IFTextField t;

void setup() {
  myTwitter = new Twitter("yourTwitterUserName", "yourTwitterPassword");
  size(600,200);
  smooth();
  font = loadFont("PalatinoLinotype-Italic-12.vlw"); 
  textFont(font); 

  searchString = "anderssandberg";
  c = new GUIController(this);
  t = new IFTextField(searchString, 15, 5, 150);
  c.add(t); 
  t.addActionListener(this);
  loading = false;
  
  // 40 as initial threshold for showing arc
  showThreshold = 40;
  pageLimit = 1;
  
  getData();
}
  
void getData()
{
  loading = true;
  
  times = new float[Nmax*maxPages+1];
  persons = new ArrayList();
  msgs = new ArrayList();
  dates = new ArrayList();
  try {
    Query query = new Query(searchString);
    query.setRpp(Nmax);
    
    int counter=0;
    for (int page=1; page<=pageLimit; page++)
    {
    query.setPage(page);
    QueryResult result = myTwitter.search(query);
 
    ArrayList tweets = (ArrayList) result.getTweets();
 
    for (int i = 0; i < tweets.size(); i++) {
      Tweet t = (Tweet) tweets.get(i);
      String user = t.getFromUser();
      String msg = t.getText();
      Date d = t.getCreatedAt();
      float  dd = int(d.getTime()/1000);
      //println("Tweet by " + user + " at " + dd + ": " + msg);
      //println(d.toString());
      if (counter==0) {maxtime=dd; mintime=dd; }
      times[counter++]=dd;
      persons.add(user);
      msgs.add(msg);
      dates.add(d.toString());
      if (dd>maxtime) maxtime=dd;
      if (dd<mintime) mintime=dd;
    };
  }
  }
  catch (TwitterException te) {
    println("Couldn't connect: " + te);
  };
  
  //println("Persons.size() = " + persons.size());
  
  score = new float[persons.size()][persons.size()];
  for (int i=0; i< persons.size(); i++)
    for (int j=i+1; j< persons.size(); j++)
       {
         int K=5; // length of overlapping sections
         String s1 = (String) msgs.get(i);
         String s2 = (String) msgs.get(j);
         for (int offset1 = 0; offset1<s1.length()-K; offset1++)
            for (int offset2 = 0; offset2<s2.length()-K; offset2++)
             if (s1.regionMatches(false, offset1, s2, offset2, K))
               score[i][j] += 1.0; 
       //println("Score " + score[i][j]);
       }
       loading=false; 
};
 
void draw() { 
if (loading==false)
{
  background(230);

  if (persons.size()>0)
  {
  ellipseMode(RADIUS);
   stroke(36,90,122);
   fill(9,105,162);
   for (int i=0; i< persons.size(); i++)
   {
   float x= width*(times[i]-mintime)/(maxtime-mintime);
   float  y=height * float(i)/persons.size();
   ellipse(x,y,2,2);
   }
   displayNearest();
   }
}
else
{
  background(230);
}
};

void displayNearest()
{
  int index = persons.size()*mouseY/height;

   stroke(61,154,209);
   fill(100,168,209);
   ellipse(width*(times[index]-mintime)/(maxtime-mintime),height * float(index)/persons.size(),3,3);

   rectMode(CORNER);
   fill(0,0,0);
   text((String) persons.get(index), 15, 40);
   text((String) dates.get(index), 15, 60);
   text((String) msgs.get(index),width - 215, height - 70, 200,70);
   
for (int i=0; i< persons.size(); i++)
 {
   /*
   if (((String) persons.get(i)).equals((String) persons.get(index)))
   {
   float x= width*(times[i]-mintime)/(maxtime-mintime);
   float  y=height * float(i)/persons.size();
   fill(0,255,0);
   ellipse(x,y,2,2);
   }
   */

   if (score[index][i]>showThreshold)
   {
   float x= width*(times[i]-mintime)/(maxtime-mintime);
   float  y=height * float(i)/persons.size();
   fill(9,105,162);
   ellipse(x,y,2,2);
   
   float x2= width*(times[index]-mintime)/(maxtime-mintime);
   float  y2=height * float(index)/persons.size();
   noFill();
   arc(x2,y,x2-x,y-y2,PI,3*PI/2);
   }
 }
}

void actionPerformed(GUIEvent e) {
  if (e.getMessage().equals("Completed")) {
    searchString = t.getValue();
    background(230);
    getData();
  }
}

void keyPressed()
{
  if( key == '+' && pageLimit < maxPages)
 {
   pageLimit++;
     println("Score " + pageLimit);
      getData();
}
  if (key == '-' && pageLimit > 1)
 {
   pageLimit--;
     println("Score " + pageLimit);
  getData();
}
  if (key == '/' && showThreshold > 1)
   showThreshold--;
  if (key == '*')
   showThreshold++;
}  
  


